==> v8.70.zip: wcs/tests/admin_pages/test_tests.py <==
import datetime
import json
import os

import pytest
from django.utils.html import escape
from webtest import Upload

from wcs import fields
from wcs.formdef import FormDef
from wcs.qommon.http_request import HTTPRequest
from wcs.testdef import TestDef

from ..utilities import clean_temporary_pub, create_temporary_pub, get_app, login
from .test_all import create_superuser


@pytest.fixture
def pub():
    pub = create_temporary_pub()

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.write_cfg()

    if not pub.site_options.has_section('options'):
        pub.site_options.add_section('options')
    pub.site_options.set('options', 'enable-tests', 'true')
    with open(os.path.join(pub.app_dir, 'site-options.cfg'), 'w') as fd:
        pub.site_options.write(fd)

    FormDef.wipe()
    TestDef.wipe()
    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_tests_link_on_formdef_page(pub):
    formdef = FormDef()
    formdef.name = 'test title'
    formdef.store()

    create_superuser(pub)
    app = login(get_app(pub))

    resp = app.get(formdef.get_admin_url())
    assert 'Tests' in resp.text

    pub.site_options.set('options', 'enable-tests', 'false')
    with open(os.path.join(pub.app_dir, 'site-options.cfg'), 'w') as fd:
        pub.site_options.write(fd)

    resp = app.get(formdef.get_admin_url())
    assert 'Tests' not in resp.text


def test_tests_page(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.PageField(
            id='0',
            label='1st page',
            type='page',
            post_conditions=[
                {'condition': {'type': 'django', 'value': 'form_var_text == "a"'}, 'error_message': 'Error'}
            ],
        ),
        fields.StringField(id='1', label='Text', varname='text'),
    ]
    formdef.store()

    app = login(get_app(pub))

    resp = app.get(formdef.get_admin_url())
    resp = resp.click('Tests')
    assert 'There are no tests yet.' in resp.text
    assert 'Run' not in resp.text
    assert 'Tests cannot be created because there are no completed forms.' in resp.text
    assert 'New' not in resp.text

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'a'
    formdata.user_id = user.id
    formdata.store()

    resp = app.get(formdef.get_admin_url())
    resp = resp.click('Tests')
    assert 'There are no tests yet.' in resp.text
    assert 'Run' not in resp.text
    assert 'New tests cannot be created' not in resp.text

    resp = resp.click('New')
    resp.form['name'] = 'First test'
    resp.form['formdata_id'].select(text='1-1 - admin')
    assert len(resp.form['formdata_id'].options) == 1

    resp = resp.form.submit().follow()
    assert 'First test' in resp.text
    assert 'no tests yet' not in resp.text

    resp = resp.click('Run')
    assert 'First test: Success!' in resp.text

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'b'
    formdata.store()

    resp = app.get('/backoffice/forms/1/tests/new')
    resp.form['name'] = 'Second test'
    resp.form['formdata_id'].select(text='1-2 - Unknown User')
    assert len(resp.form['formdata_id'].options) == 2

    resp = resp.form.submit().follow()
    assert 'First test' in resp.text
    assert 'Second test' in resp.text

    resp = resp.click('Run')
    assert 'First test: Success!' in resp.text
    assert 'Second test: Page 1 post condition was not met (form_var_text == &quot;a&quot;).' in resp.text


def test_tests_page_formdefs_isolation(pub):
    formdef = FormDef()
    formdef.name = 'dummy'
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.store()

    other_formdef = FormDef()
    other_formdef.name = 'dummy2'
    other_formdef.store()

    formdata2 = other_formdef.data_class()()
    formdata2.just_created()
    formdata2.store()

    create_superuser(pub)
    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/new')
    assert len(resp.form['formdata_id'].options) == 1
    assert resp.form['formdata_id'].options[0][2] == '1-1 - Unknown User'

    resp = app.get('/backoffice/forms/2/tests/new')
    assert len(resp.form['formdata_id'].options) == 1
    assert resp.form['formdata_id'].options[0][2] == '2-1 - Unknown User'


def test_tests_import_export(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [fields.StringField(id='1', varname='test field', label='Test', type='string')]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'a'
    formdata.user_id = user.id
    formdata.store()

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.name = 'First test'
    testdef.store()

    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/')
    export_resp = resp.click('First test')

    resp = resp.click('remove')
    resp = resp.form.submit().follow()
    assert 'First test' not in resp.text

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', export_resp.body, 'application/json')
    resp = resp.form.submit().follow()
    assert TestDef.count() == 1
    assert 'First test' in resp.text
    assert escape('Test "First test" has been successfully imported.') in resp.text

    imported_testdef = TestDef.select()[0]
    assert imported_testdef.export_to_json() == testdef.export_to_json()

    export_json = json.loads(export_resp.body)
    export_json['data']['fields']['1'] = 'b'

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', json.dumps(export_json).encode(), 'application/json')
    resp = resp.form.submit().follow()
    assert TestDef.count() == 1
    assert 'First test' in resp.text
    assert escape('Test "First test" has been successfully imported.') in resp.text

    imported_testdef = TestDef.get(imported_testdef.id)
    assert imported_testdef.data['fields']['1'] == 'b'

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', b'invalid', 'application/json')
    resp = resp.form.submit()
    assert 'Expecting value: line 1 column 1' in resp.text

==> v8.71.zip: wcs/tests/admin_pages/test_tests.py <==
import datetime
import json
import os

import pytest
from django.utils.html import escape
from webtest import Upload

from wcs import fields
from wcs.formdef import FormDef
from wcs.qommon.http_request import HTTPRequest
from wcs.testdef import TestDef

from ..utilities import clean_temporary_pub, create_temporary_pub, get_app, login
from .test_all import create_superuser


@pytest.fixture
def pub():
    pub = create_temporary_pub()

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.write_cfg()

    if not pub.site_options.has_section('options'):
        pub.site_options.add_section('options')
    pub.site_options.set('options', 'enable-tests', 'true')
    with open(os.path.join(pub.app_dir, 'site-options.cfg'), 'w') as fd:
        pub.site_options.write(fd)

    FormDef.wipe()
    TestDef.wipe()
    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_tests_link_on_formdef_page(pub):
    formdef = FormDef()
    formdef.name = 'test title'
    formdef.store()

    create_superuser(pub)
    app = login(get_app(pub))

    resp = app.get(formdef.get_admin_url())
    assert 'Tests' in resp.text

    pub.site_options.set('options', 'enable-tests', 'false')
    with open(os.path.join(pub.app_dir, 'site-options.cfg'), 'w') as fd:
        pub.site_options.write(fd)

    resp = app.get(formdef.get_admin_url())
    assert 'Tests' not in resp.text


def test_tests_page(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.PageField(
            id='0',
            label='1st page',
            type='page',
            post_conditions=[
                {'condition': {'type': 'django', 'value': 'form_var_text == "a"'}, 'error_message': 'Error'}
            ],
        ),
        fields.StringField(id='1', label='Text', varname='text'),
    ]
    formdef.store()

    app = login(get_app(pub))

    resp = app.get(formdef.get_admin_url())
    resp = resp.click('Tests')
    assert 'There are no tests yet.' in resp.text
    assert 'Run' not in resp.text
    assert 'Tests cannot be created because there are no completed forms.' in resp.text
    assert 'New' not in resp.text

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'a'
    formdata.user_id = user.id
    formdata.store()

    resp = app.get(formdef.get_admin_url())
    resp = resp.click('Tests')
    assert 'There are no tests yet.' in resp.text
    assert 'Run' not in resp.text
    assert 'New tests cannot be created' not in resp.text

    resp = resp.click('New')
    resp.form['name'] = 'First test'
    resp.form['formdata_id'].select(text='1-1 - admin')
    assert len(resp.form['formdata_id'].options) == 1

    resp = resp.form.submit().follow()
    assert 'First test' in resp.text
    assert 'no tests yet' not in resp.text

    resp = resp.click('Run')
    assert 'First test: Success!' in resp.text

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'b'
    formdata.store()

    resp = app.get('/backoffice/forms/1/tests/new')
    resp.form['name'] = 'Second test'
    resp.form['formdata_id'].select(text='1-2 - Unknown User')
    assert len(resp.form['formdata_id'].options) == 2

    resp = resp.form.submit().follow()
    assert 'First test' in resp.text
    assert 'Second test' in resp.text

    resp = resp.click('Run')
    assert 'First test: Success!' in resp.text
    assert 'Second test: Page 1 post condition was not met (form_var_text == &quot;a&quot;).' in resp.text


def test_tests_page_formdefs_isolation(pub):
    formdef = FormDef()
    formdef.name = 'dummy'
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.store()

    other_formdef = FormDef()
    other_formdef.name = 'dummy2'
    other_formdef.store()

    formdata2 = other_formdef.data_class()()
    formdata2.just_created()
    formdata2.store()

    create_superuser(pub)
    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/new')
    assert len(resp.form['formdata_id'].options) == 1
    assert resp.form['formdata_id'].options[0][2] == '1-1 - Unknown User'

    resp = app.get('/backoffice/forms/2/tests/new')
    assert len(resp.form['formdata_id'].options) == 1
    assert resp.form['formdata_id'].options[0][2] == '2-1 - Unknown User'


def test_tests_import_export(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [fields.StringField(id='1', varname='test field', label='Test', type='string')]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'a'
    formdata.user_id = user.id
    formdata.store()

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.name = 'First test'
    testdef.store()

    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/%s/' % testdef.id)
    export_resp = resp.click('Export')

    resp = resp.click('Delete')
    resp = resp.form.submit().follow()
    assert 'First test' not in resp.text

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', export_resp.body, 'application/json')
    resp = resp.form.submit().follow()
    assert TestDef.count() == 1
    assert 'First test' in resp.text
    assert escape('Test "First test" has been successfully imported.') in resp.text

    imported_testdef = TestDef.select()[0]
    assert imported_testdef.export_to_json() == testdef.export_to_json()

    export_json = json.loads(export_resp.body)
    export_json['data']['fields']['1'] = 'b'

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', json.dumps(export_json).encode(), 'application/json')
    resp = resp.form.submit().follow()
    assert TestDef.count() == 1
    assert 'First test' in resp.text
    assert escape('Test "First test" has been successfully imported.') in resp.text

    imported_testdef = TestDef.get(imported_testdef.id)
    assert imported_testdef.data['fields']['1'] == 'b'

    resp = resp.click('Import')
    resp.form['file'] = Upload('export.json', b'invalid', 'application/json')
    resp = resp.form.submit()
    assert 'Expecting value: line 1 column 1' in resp.text


def test_tests_status_page(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [fields.StringField(id='1', varname='test_field', label='Test Field', type='string')]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'This is a test'
    formdata.user_id = user.id
    formdata.store()

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.name = 'First test'
    testdef.store()

    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/')
    resp = resp.click('First test')
    assert 'Test Field' in resp.text
    assert 'This is a test' in resp.text

    resp = resp.click('Inspect')
    resp.form['django-condition'] = 'form_var_test_field == "This is a test"'
    resp = resp.form.submit()
    assert 'Condition result' in resp.text
    assert 'result-true' in resp.text


def test_tests_edit(pub):
    user = create_superuser(pub)

    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.PageField(id='0', label='1st page', type='page'),
        fields.StringField(id='1', label='Text 1', varname='text1'),
        fields.PageField(id='2', label='2nd page', type='page'),
        fields.StringField(id='3', label='Text 2', varname='text2'),
    ]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'aaa'
    formdata.data['3'] = 'bbb'
    formdata.user_id = user.id
    formdata.store()

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.name = 'First test'
    testdef.store()

    app = login(get_app(pub))

    resp = app.get('/backoffice/forms/1/tests/%s/' % testdef.id)
    assert 'aaa' in resp.text
    assert 'bbb' in resp.text

    resp = resp.click('Edit')
    resp.form['f1'] = 'xxx'
    resp = resp.form.submit('submit')
    resp = resp.form.submit('submit').follow()  # change nothing on second page
    assert 'aaa' not in resp.text
    assert 'xxx' in resp.text
    assert 'bbb' in resp.text
